@@ -15,21 +15,23 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- * May, 2000. Implemented on FreeBSD 4.0R (Compaq Armada M700)
- *
- $Id: battstat.h,v 1.29 2005/03/20 10:20:55 kmaraas Exp $
+ $Id: battstat.h,v 1.32 2005/06/10 08:20:59 davyd Exp $
  */
 
 
 #include <glib.h>
-#include <gdk/gdkgc.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 
 #define DEBUG 0
 
 #define PROGLEN 33.0
 
+/* I made these multipliers up
+ *  --davyd
+ */
+#define ORANGE_MULTIPLIER 1.5
+#define YELLOW_MULTIPLIER 2.5
+
 typedef enum
 {
   APPLET_SHOW_NONE,
@@ -40,8 +42,9 @@ typedef enum
 typedef enum
 {
   STATUS_PIXMAP_BATTERY,
+  STATUS_PIXMAP_METER,
   STATUS_PIXMAP_AC,
-  STATUS_PIXMAP_FLASH,
+  STATUS_PIXMAP_CHARGE,
   STATUS_PIXMAP_WARNING,
   STATUS_PIXMAP_NUM
 } StatusPixmapIndex;
@@ -87,19 +90,25 @@ typedef struct _ProgressData {
   GtkWidget *applet;
 
   /* these are used by properties.c */
-  GtkWidget *radio_lay_batt_on;
-  GtkWidget *radio_lay_status_on;
+  GtkWidget *radio_ubuntu_battery;
+  GtkWidget *radio_traditional_battery;
   GtkWidget *radio_text_1;
   GtkWidget *radio_text_2;
   GtkWidget *check_text;
   GtkWidget *suspend_entry;
   GtkWidget *lowbatt_toggle;
   GtkWidget *full_toggle;
+  
+  GtkWidget *hbox_ptr;
+  GtkWidget *combo_ptr;
+  GtkWidget *spin_ptr;
+  GtkWidget *label_ptr;
 
   /* flags set from gconf or the properties dialog */
   guint red_val;
   guint orange_val;
   guint yellow_val;
+  gboolean red_value_is_time;
   gboolean lowbattnotification;
   gboolean fullbattnot;
   gboolean beep;
@@ -127,9 +136,12 @@ typedef struct _ProgressData {
   GtkWidget *percent;
 
   /* dialog boxes that might be displayed */
-  GtkWidget *lowbattnotificationdialog;
   GtkWidget *about_dialog;
   GtkDialog *prop_win;
+  GtkWidget *battery_low_dialog;
+
+  /* text label inside the low battery dialog */
+  GtkLabel *battery_low_label;
 
   /* our height/width as given to us by size_allocate */
   gint width, height;
