@@ -16,9 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- * May, 2000. Implemented on FreeBSD 4.0-RELEASE (Compaq Armada M700)
- *
- $Id: battstat_applet.c,v 1.99 2005/03/20 10:20:55 kmaraas Exp $
+ $Id: battstat_applet.c,v 1.105 2005/06/19 10:53:56 davyd Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,22 +31,21 @@
 #include <time.h>
 #include <unistd.h>
 
-#include <gdk/gdkkeysyms.h>
-#include <gtk/gtkaccessible.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkicontheme.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
+#include <gtk/gtk.h>
+
+#include <libgnome/libgnome.h>
+
+#include <libgnomeui/libgnomeui.h>
 
 #include <glade/glade.h>
 
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 
+#ifdef HAVE_LIBNOTIFY
+#include <libnotify/notify.h>
+#endif
+
 #include "battstat.h"
 #include "pixmaps.h"
 
@@ -242,13 +239,17 @@ initialise_global_pixmaps( void )
     gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_BATTERY],
                                   NULL, battery_small_xpm );
 
+  statusimage[STATUS_PIXMAP_METER] =
+    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_METER],
+                                  NULL, battery_small_meter_xpm );
+
   statusimage[STATUS_PIXMAP_AC] =
     gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_AC],
                                   NULL, ac_small_xpm );
    
-  statusimage[STATUS_PIXMAP_FLASH] =
-    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_FLASH],
-                                  NULL, flash_small_xpm );
+  statusimage[STATUS_PIXMAP_CHARGE] =
+    gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_CHARGE],
+                                  NULL, charge_small_xpm );
    
   statusimage[STATUS_PIXMAP_WARNING] =
     gdk_pixmap_create_from_xpm_d( defaults, &statusmask[STATUS_PIXMAP_WARNING],
@@ -410,11 +411,45 @@ get_remaining (BatteryStatus *info)
 						info->percent);
 }
 
+static gboolean
+battery_full_notify (void)
+{
+#ifdef HAVE_LIBNOTIFY
+	static NotifyIcon *icon = NULL;
+	
+	if (!notify_is_initted () && !notify_init (_("Battery Monitor")))
+		return FALSE;
+
+	/* XXX: this icon is not found in the theme... strange */
+	/* if (!icon)
+		icon = notify_icon_new_from_uri ("gnome-dev-battery"); */
+	
+	if (!notify_send_notification (NULL,
+				"device",
+				NOTIFY_URGENCY_NORMAL,
+				_("Your battery is now fully recharged"),
+				NULL,		/* body text */
+				icon,		/* icon */
+				TRUE, 0,	/* expiry, server default */
+				NULL,		/* no user_data */
+				0))		/* no actions */
+		return FALSE;
+
+	return TRUE;
+#else
+	return FALSE;
+#endif
+}
+
 /* Show a dialog notifying the user that their battery is done charging.
  */
 static void
 battery_full_dialog( void )
 {
+  /* first attempt to use libnotify */
+  if (battery_full_notify ())
+	  return;
+  
   GtkWidget *dialog, *hbox, *image, *label;
   GdkPixbuf *pixbuf;
 
@@ -461,10 +496,61 @@ battery_full_dialog( void )
 /* Destroy the low battery notification dialog and mark it as such.
  */
 static void
-on_lowbatt_notification_response( GtkWidget *w, gint a, GtkWidget **self )
+battery_low_dialog_destroy( ProgressData *battstat )
 {
-  gtk_widget_destroy( GTK_WIDGET (*self) );
-  *self = NULL;
+  gtk_widget_destroy( battstat->battery_low_dialog );
+  battstat->battery_low_dialog = NULL;
+  battstat->battery_low_label = NULL;
+}
+
+/* Update the text label in the battery low dialog.
+ */
+static void
+battery_low_update_text( ProgressData *battstat, BatteryStatus *info )
+{
+  gchar *remaining, *new_label;
+  GtkRequisition size;
+
+  /* If we're not displaying the dialog then don't update it. */
+  if( battstat->battery_low_label == NULL ||
+      battstat->battery_low_dialog == NULL )
+    return;
+
+  gtk_widget_size_request( GTK_WIDGET( battstat->battery_low_label ), &size );
+
+  /* If the label has never been set before, the width will be 0.  If it
+     has been set before (width > 0) then we want to keep the size of
+     the old widget (to keep the dialog from changing sizes) so we set it
+     explicitly here.
+   */
+  if( size.width > 0 )
+    gtk_widget_set_size_request( GTK_WIDGET( battstat->battery_low_label ),
+                                 size.width, size.height );
+
+  remaining = g_strdup_printf( ngettext(
+                                 "You have %d minute of battery power "
+				   "remaining (%d%% of the total capacity).",
+                                 "You have %d minutes of battery power "
+				   "remaining (%d%% of the total capacity).",
+                                 info->minutes ),
+                               info->minutes,info->percent );
+
+  new_label = g_strdup_printf (
+		"<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n\n%s",
+		_("Your battery is running low"),
+		remaining,
+		/* TRANSLATORS: this is a list, it is left as a single string
+		 * to allow you to make it appear like a list would in your
+		 * locale. */
+		_("To avoid losing your work:\n"
+		  " \xE2\x80\xA2 suspend your laptop to save power,\n"
+		  " \xE2\x80\xA2 plug your laptop into external power, or\n"
+		  " \xE2\x80\xA2 save open documents and shut your laptop down."
+		  ));
+
+  gtk_label_set_markup( battstat->battery_low_label, new_label );
+  g_free( remaining );
+  g_free( new_label );
 }
 
 /* Show a dialog notifying the user that their battery is running low.
@@ -473,30 +559,32 @@ static void
 battery_low_dialog( ProgressData *battery, BatteryStatus *info )
 {
   GtkWidget *hbox, *image, *label;
-  gchar *new_string, *new_label;
+  GtkWidget *vbox;
   GdkPixbuf *pixbuf;
 
-  new_string = get_remaining (info);
-  new_label = g_strdup_printf (
-		"<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s. %s",
-		_("Your battery is running low"),
-		new_string,
-		_("To avoid losing work please power off, suspend or plug your laptop in."));
-  g_free (new_string);
-  battery->lowbattnotificationdialog = gtk_dialog_new_with_buttons (
+  /* If the dialog is already displayed then don't display it again. */
+  if( battery->battery_low_dialog != NULL )
+    return;
+
+  battery->battery_low_dialog = gtk_dialog_new_with_buttons (
 		 _("Battery Notice"),
 		 NULL,
 		 GTK_DIALOG_DESTROY_WITH_PARENT,
 		 GTK_STOCK_OK,
 		 GTK_RESPONSE_ACCEPT,
 		 NULL);
-  g_signal_connect (G_OBJECT (battery->lowbattnotificationdialog), "response",
-		    G_CALLBACK (on_lowbatt_notification_response),
-		    &battery->lowbattnotificationdialog);
 
-  gtk_container_set_border_width (GTK_CONTAINER (battery->lowbattnotificationdialog), 6);
-  gtk_dialog_set_has_separator (GTK_DIALOG (battery->lowbattnotificationdialog), FALSE);
+  g_signal_connect_swapped( GTK_OBJECT (battery->battery_low_dialog),
+                            "response",
+                            G_CALLBACK (battery_low_dialog_destroy),
+                            battery );
+
+  gtk_container_set_border_width (GTK_CONTAINER (battery->battery_low_dialog),
+		  6);
+  gtk_dialog_set_has_separator (GTK_DIALOG (battery->battery_low_dialog),
+		  FALSE);
   hbox = gtk_hbox_new (FALSE, 6);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
   pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 		 "gnome-dev-battery",
 		 48,
@@ -504,23 +592,26 @@ battery_low_dialog( ProgressData *battery, BatteryStatus *info )
 		 NULL);
   image = gtk_image_new_from_pixbuf (pixbuf);
   g_object_unref (pixbuf);
-  gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 6);
-  label = gtk_label_new (new_label);
-  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-  g_free (new_label);
-  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 6);
-  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (battery->lowbattnotificationdialog)->vbox), hbox);
+  vbox = gtk_vbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (vbox), image, FALSE, FALSE, 0);
+  label = gtk_label_new ("");
+  battery->battery_low_label = GTK_LABEL( label );
+  gtk_label_set_line_wrap( battery->battery_low_label, TRUE );
+  gtk_label_set_selectable( battery->battery_low_label, TRUE );
+  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 6);
+  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (battery->battery_low_dialog)->vbox), hbox);
 	 
-  gtk_window_set_keep_above (GTK_WINDOW (battery->lowbattnotificationdialog), TRUE);
-  gtk_window_stick (GTK_WINDOW (battery->lowbattnotificationdialog));
-  gtk_window_set_focus_on_map (GTK_WINDOW (battery->lowbattnotificationdialog),
+  gtk_window_set_keep_above (GTK_WINDOW (battery->battery_low_dialog), TRUE);
+  gtk_window_stick (GTK_WINDOW (battery->battery_low_dialog));
+  gtk_window_set_focus_on_map (GTK_WINDOW (battery->battery_low_dialog),
 		  FALSE);
-  gtk_window_set_skip_pager_hint (GTK_WINDOW (
-			  battery->lowbattnotificationdialog),
+  gtk_window_set_skip_pager_hint (GTK_WINDOW (battery->battery_low_dialog),
 		  TRUE);
-  gtk_widget_show_all (battery->lowbattnotificationdialog);
-  battery->lowbattnotification=FALSE;
+
+  battery_low_update_text( battery, info );
+
+  gtk_widget_show_all (battery->battery_low_dialog);
 }
 
 /* Update the text of the tooltip from the provided info.
@@ -561,27 +652,37 @@ update_tooltip( ProgressData *battstat, BatteryStatus *info )
 /* Redraw the battery meter image.
  */
 static void
-update_battery_image( ProgressData *battstat, int batt_life )
+update_battery_image (ProgressData *battstat, int batt_percent, int batt_time)
 {
   GdkColor *color, *darkcolor;
   GdkPixmap *pixmap;
   GdkBitmap *pixmask;
   guint progress_value;
   gint i, x;
+  int batt_life;
 
   if (!battstat->showbattery)
     return;
 
-  if (batt_life <= battstat->red_val) {
+  batt_life = !battstat->red_value_is_time ? batt_percent : batt_time;
+
+  if (batt_life <= battstat->red_val)
+  {
     color = red;
     darkcolor = darkred;
-  } else if (batt_life <= battstat->orange_val) {
+  }
+  else if (batt_life <= battstat->orange_val)
+  {
     color = orange;
     darkcolor = darkorange;
-  } else if (batt_life <= battstat->yellow_val) {
+  }
+  else if (batt_life <= battstat->yellow_val)
+  {
     color = yellow;
     darkcolor = darkyellow;
-  } else {
+  }
+  else
+  {
     color = green;
     darkcolor = darkgreen;
   }
@@ -706,6 +807,23 @@ update_percent_label( ProgressData *battstat, BatteryStatus *info )
   g_free (new_label);
 }
 
+/* Utility function to create a copy of a GdkPixmap */
+static GdkPixmap *
+copy_gdk_pixmap( GdkPixmap *src, GdkGC *gc )
+{
+  gint height, width;
+  GdkPixmap *dest;
+
+  gdk_drawable_get_size( GDK_DRAWABLE( src ), &width, &height );
+
+  dest = gdk_pixmap_new( GDK_DRAWABLE( src ), width, height, -1 );
+
+  gdk_draw_drawable( GDK_DRAWABLE( dest ), gc, GDK_DRAWABLE( src ),
+                     0, 0, 0, 0, width, height );
+
+  return dest;
+}
+
 /* Determine what status icon we ought to be displaying and change the
    status icon to display it if it is different from what we are currently
    showing.
@@ -714,24 +832,106 @@ static void
 possibly_update_status_icon( ProgressData *battstat, BatteryStatus *info )
 {
   StatusPixmapIndex pixmap_index;
+  int batt_life, last_batt_life;
+
+  batt_life = !battstat->red_value_is_time ? info->percent : info->minutes;
+  last_batt_life = !battstat->red_value_is_time ? battstat->last_batt_life :
+	  					 battstat->last_minutes;
 
   if( info->on_ac_power )
   {
-    /* only show 'flash' if charging and it wasn't shown last time */
-    if( info->charging && battstat->last_pixmap_index == STATUS_PIXMAP_AC )
-      pixmap_index = STATUS_PIXMAP_FLASH;
+    if( info->charging )
+      pixmap_index = STATUS_PIXMAP_CHARGE;
     else
       pixmap_index = STATUS_PIXMAP_AC;
   }
   else /* on battery */
   {
-    if( info->percent > battstat->red_val )
+    if (batt_life > battstat->red_val)
       pixmap_index = STATUS_PIXMAP_BATTERY;
     else
       pixmap_index = STATUS_PIXMAP_WARNING;
   }
 
-  if ( pixmap_index != battstat->last_pixmap_index )
+  /* If we are showing the full length battery meter then the status icon
+     should display static icons.  If we are not showing the full meter
+     then the status icon will show a smaller meter if we are on battery.
+   */
+  if( !battstat->showbattery && 
+      (pixmap_index == STATUS_PIXMAP_BATTERY ||
+       pixmap_index == STATUS_PIXMAP_WARNING) )
+    pixmap_index = STATUS_PIXMAP_METER;
+
+
+  /* Take care of drawing the smaller meter. */
+  if( pixmap_index == STATUS_PIXMAP_METER &&
+      (batt_life != last_batt_life ||
+       battstat->last_pixmap_index != STATUS_PIXMAP_METER) )
+  {
+    GdkColor *colour;
+    GdkPixmap *meter;
+    guint progress_value;
+    gint i, x;
+
+    /* We keep this pixgc allocated so we don't have to alloc/free it every
+       time.  A widget has to be realized before it has a valid ->window so
+       we do that here for battstat->applet just in case it's not already done.
+    */
+    if( battstat->pixgc == NULL )
+    {
+      gtk_widget_realize( battstat->applet );
+      battstat->pixgc = gdk_gc_new( battstat->applet->window );
+    }
+
+    /* Pull in a clean version of the icons so that we don't paint over
+       top of the same icon over and over.  We neglect to free/update the
+       statusmask here since it will always stay the same.
+     */
+    meter = copy_gdk_pixmap( statusimage[STATUS_PIXMAP_METER],
+                             battstat->pixgc );
+
+    if (batt_life <= battstat->red_val)
+    {
+      colour = red;
+    }
+    else if (batt_life <= battstat->orange_val)
+    {
+      colour = orange;
+    }
+    else if (batt_life <= battstat->yellow_val)
+    {
+      colour = yellow;
+    }
+    else
+    {
+      colour = green;
+    }
+
+    progress_value = 12 * info->percent / 100.0;
+    
+    for( i = 0; i < 10; i++ )
+    {
+      gdk_gc_set_foreground( battstat->pixgc, &colour[(i * 13 / 10)] );
+
+      if( i >= 2 && i <= 7 )
+	x = 17;
+      else
+	x = 16;
+
+      gdk_draw_line( meter, battstat->pixgc,
+		     i + 1, x - progress_value,
+		     i + 1, x );
+    }
+
+    /* force a redraw immediately */
+    gtk_image_set_from_pixmap( GTK_IMAGE (battstat->status),
+                               meter, statusmask[STATUS_PIXMAP_METER] );
+
+    /* free our private pixmap copy */
+    g_object_unref( G_OBJECT( meter ) );
+    battstat->last_pixmap_index = STATUS_PIXMAP_METER;
+  }
+  else if( pixmap_index != battstat->last_pixmap_index )
   {
     gtk_image_set_from_pixmap (GTK_IMAGE (battstat->status),
                                statusimage[pixmap_index],
@@ -759,8 +959,16 @@ check_for_updates( gpointer data )
 
   if (!info.on_ac_power &&
       battstat->last_batt_life != 1000 &&
-      battstat->last_batt_life > battstat->red_val &&
-      info.percent <= battstat->red_val &&
+      (
+        /* if percentage drops below red_val */
+        !battstat->red_value_is_time &&
+        battstat->last_batt_life > battstat->red_val &&
+        info.percent <= battstat->red_val ||
+	/* if time drops below red_val */
+	battstat->red_value_is_time &&
+	battstat->last_minutes > battstat->red_val &&
+	info.minutes <= battstat->red_val
+      ) &&
       info.present)
   {
     /* Warn that battery dropped below red_val */
@@ -795,26 +1003,11 @@ check_for_updates( gpointer data )
     }
   }
 
-  if( !battstat->last_charging &&
-      info.charging &&
-      info.on_ac_power &&
-      info.present)
-  {
-    /*
-     * the battery is charging again, reset the dialog display flag
-     * Thanks to Richard Kinder <r_kinder@yahoo.com> for this patch.
-     */
-    battstat->lowbattnotification = panel_applet_gconf_get_bool (
-                           PANEL_APPLET(battstat->applet),
-                           GCONF_PATH "low_battery_notification", NULL);
-
-    if (battstat->lowbattnotificationdialog)
-    {
-      /* we can remove the battery warning dialog */
-      gtk_widget_destroy (battstat->lowbattnotificationdialog);
-      battstat->lowbattnotificationdialog = NULL;
-    }
-  }
+  /* If the warning dialog is displayed and we just got plugged in then
+     stop displaying it.
+   */
+  if( battstat->battery_low_dialog && info.on_ac_power )
+    battery_low_dialog_destroy( battstat );
 
   if( info.on_ac_power != battstat->last_acline_status ||
       info.percent != battstat->last_batt_life ||
@@ -824,12 +1017,16 @@ check_for_updates( gpointer data )
   {
     /* Update the tooltip */
     update_tooltip( battstat, &info );
+
+    /* If the warning dialog box is currently displayed, update that too. */
+    if( battstat->battery_low_dialog != NULL )
+      battery_low_update_text( battstat, &info );
   }
 
   if( info.percent != battstat->last_batt_life )
   {
     /* Update the battery meter image */
-    update_battery_image( battstat, info.percent );
+    update_battery_image (battstat, info.percent, info.minutes);
   }
 
   if( (battstat->showtext == APPLET_SHOW_PERCENT &&
@@ -848,6 +1045,7 @@ check_for_updates( gpointer data )
   battstat->last_charging = info.charging;
   battstat->last_batt_state = info.state;
   battstat->last_batt_life = info.percent;
+  battstat->last_minutes = info.minutes;
   battstat->last_acline_status = info.on_ac_power;
 
   return TRUE;
@@ -868,8 +1066,8 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
   if (battstat->prop_win)
     gtk_widget_destroy (GTK_WIDGET (battstat->prop_win));
 
-  if( battstat->lowbattnotificationdialog )
-    gtk_widget_destroy( battstat->lowbattnotificationdialog );
+  if( battstat->battery_low_dialog )
+    battery_low_dialog_destroy( battstat );
 
   gtk_timeout_remove( battstat->pixtimer );
 
@@ -890,22 +1088,24 @@ destroy_applet( GtkWidget *widget, ProgressData *battstat )
 /* Called when the user selects the 'help' menu item.
  */
 static void
-help_cb (BonoboUIComponent *uic,
-	 ProgressData      *battstat,
-	 const char        *verb)
+help_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
 {
-    GError *error = NULL;
+  GError *error = NULL;
 
-    gnome_help_display_on_screen (
-		"battstat", NULL,
-		gtk_widget_get_screen (battstat->applet),
-		&error);
+  gnome_help_display_on_screen( "battstat", NULL,
+		                gtk_widget_get_screen( battstat->applet ),
+		                &error );
 
-    if (error) { /* FIXME: the user needs to see this */
-        g_warning ("help error: %s\n", error->message);
-        g_error_free (error);
-        error = NULL;
-    }
+  if( error )
+  {
+    char *message;
+
+    message = g_strdup_printf( _("There was an error displaying help: %s"),
+                               error->message );
+    battstat_error_dialog( battstat->applet, message );
+    g_error_free( error );
+    g_free( message );
+  }
 }
 
 /* Called when the user chooses the 'suspend' menu item or double-clicks.
@@ -951,40 +1151,37 @@ suspend_cb (BonoboUIComponent *uic,
 /* Called when the user selects the 'about' menu item.
  */
 static void
-about_cb (BonoboUIComponent *uic,
-	  ProgressData      *battstat,
-	  const char        *verb)
+about_cb( BonoboUIComponent *uic, ProgressData *battstat, const char *verb )
 {
-   const gchar *authors[] = {
-	/* if your charset supports it, please replace the "o" in
-	 * "Jorgen" into U00F6 */
-	_("Jorgen Pehrson <jp@spektr.eu.org>"), 
-	"Lennart Poettering <lennart@poettering.de> (Linux ACPI support)",
-	"Seth Nickell <snickell@stanford.edu> (GNOME2 port)",
-	"Davyd Madeley <davyd@madeley.id.au>",
-	"Ryan Lortie <desrt@desrt.ca>",
-	NULL
+  const gchar *authors[] = {
+    "J\xC3\xB6rgen Pehrson <jp@spektr.eu.org>", 
+    "Lennart Poettering <lennart@poettering.de> (Linux ACPI support)",
+    "Seth Nickell <snickell@stanford.edu> (GNOME2 port)",
+    "Davyd Madeley <davyd@madeley.id.au>",
+    "Ryan Lortie <desrt@desrt.ca>",
+    "Joe Marcus Clarke <marcus@FreeBSD.org> (FreeBSD ACPI support)",
+    NULL
    };
 
-   const gchar *documenters[] = {
-        "Jorgen Pehrson <jp@spektr.eu.org>",
-        "Trevor Curtis <tcurtis@somaradio.ca>",
-	NULL
-   };
+  const gchar *documenters[] = {
+    "J\xC3\xB6rgen Pehrson <jp@spektr.eu.org>",
+    "Trevor Curtis <tcurtis@somaradio.ca>",
+    NULL
+  };
 
-   gtk_show_about_dialog (NULL,
-	"name",		_("Battery Charge Monitor"), 
-	"version",	VERSION,
-	"copyright",	"\xC2\xA9 2000 The Gnulix Society, "
-			"\xC2\xA9 2002-2005 Free Software Foundation and "
-			"others",
-	"comments",	_("This utility shows the status of your laptop "
-			  "battery."),
-	"authors",	authors,
-	"documenters",	documenters,
-	"translator-credits",	_("translator-credits"),
-	"logo-icon-name",	"gnome-dev-battery",
-	NULL);
+  gtk_show_about_dialog( NULL,
+    "name",                _("Battery Charge Monitor"), 
+    "version",             VERSION,
+    "copyright",           "\xC2\xA9 2000 The Gnulix Society, "
+                           "\xC2\xA9 2002-2005 Free Software Foundation and "
+                           "others",
+    "comments",            _("This utility shows the status of your laptop "
+                             "battery."),
+    "authors",             authors,
+    "documenters",         documenters,
+    "translator-credits",  _("translator-credits"),
+    "logo-icon-name",      "gnome-dev-battery",
+    NULL );
 }
 
 /* This signal is delivered by the panel when the orientation of the applet
@@ -1122,10 +1319,17 @@ load_preferences(ProgressData *battstat)
   
   battstat->red_val = panel_applet_gconf_get_int (applet, GCONF_PATH "red_value", NULL);
   battstat->red_val = CLAMP (battstat->red_val, 0, 100);
-  battstat->orange_val = panel_applet_gconf_get_int (applet, GCONF_PATH "orange_value", NULL);
+  battstat->red_value_is_time = panel_applet_gconf_get_bool (applet,
+		  GCONF_PATH "red_value_is_time",
+		  NULL);
+
+  /* automatically calculate orangle and yellow values from the red value */
+  battstat->orange_val = battstat->red_val * ORANGE_MULTIPLIER;
   battstat->orange_val = CLAMP (battstat->orange_val, 0, 100);
-  battstat->yellow_val = panel_applet_gconf_get_int (applet, GCONF_PATH "yellow_value", NULL);
+  
+  battstat->yellow_val = battstat->red_val * YELLOW_MULTIPLIER;
   battstat->yellow_val = CLAMP (battstat->yellow_val, 0, 100);
+
   battstat->lowbattnotification = panel_applet_gconf_get_bool (applet, GCONF_PATH "low_battery_notification", NULL);
   battstat->fullbattnot = panel_applet_gconf_get_bool (applet, GCONF_PATH "full_battery_notification", NULL);
   battstat->beep = panel_applet_gconf_get_bool (applet, GCONF_PATH "beep", NULL);
@@ -1133,6 +1337,11 @@ load_preferences(ProgressData *battstat)
   
   battstat->showstatus = panel_applet_gconf_get_bool (applet, GCONF_PATH "show_status", NULL);
   battstat->showbattery = panel_applet_gconf_get_bool (applet, GCONF_PATH "show_battery", NULL);
+
+  /* for miagration from older versions */
+  if (battstat->showstatus && battstat->showbattery)
+	  battstat->showbattery = FALSE;
+  
   battstat->showtext = panel_applet_gconf_get_int (applet, GCONF_PATH "show_text", NULL);
   battstat->suspend_cmd = panel_applet_gconf_get_string (applet, GCONF_PATH "suspend_command", NULL);
   
@@ -1234,7 +1443,7 @@ reconfigure_layout( ProgressData *battstat )
     {
        c.battery = LAYOUT_LONG;
 
-      if( battstat->showstatus )
+      /* if( battstat->showstatus ) */ /* make this always true */
         c.status = LAYOUT_TOPLEFT;
 
       if( battstat->showtext )
@@ -1257,7 +1466,7 @@ reconfigure_layout( ProgressData *battstat )
     if( up_down_order )
     {
       /* Stack horizontally for top and bottom panels. */
-      if( battstat->showstatus )
+      /* if( battstat->showstatus ) */ /* make this always true */
         c.status = LAYOUT_LEFT;
       if( battstat->showbattery )
         c.battery = LAYOUT_CENTRE;
@@ -1269,7 +1478,7 @@ reconfigure_layout( ProgressData *battstat )
     else
     {
       /* Stack vertically for left and right panels. */
-      if( battstat->showstatus )
+      /* if( battstat->showstatus ) */ /* make this always true */
         c.status = LAYOUT_TOP;
       if( battstat->showbattery )
         c.battery = LAYOUT_CENTRE;
@@ -1311,7 +1520,8 @@ reconfigure_layout( ProgressData *battstat )
        battery_horiz != battstat->horizont )
   {
     battstat->horizont = battery_horiz;
-    update_battery_image( battstat, battstat->last_batt_life );
+    update_battery_image (battstat,
+		    battstat->last_batt_life, battstat->last_minutes);
   }
 
   battstat->layout = c;
@@ -1436,7 +1646,8 @@ battstat_applet_fill (PanelApplet *applet)
   battstat->suspend_cmd = NULL;
   battstat->orienttype = panel_applet_get_orient (applet);
   battstat->horizont = TRUE;
-  battstat->lowbattnotificationdialog = NULL;
+  battstat->battery_low_dialog = NULL;
+  battstat->battery_low_label = NULL;
   battstat->about_dialog = NULL;
   battstat->pixgc = NULL;
 
