@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2001, 2002 Free Software Foundation
+ * GNOME CPUFreq Applet
+ * Copyright (C) 2004 Carlos Garcia Campos <carlosgc@gnome.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public
@@ -23,6 +24,8 @@
 #include <libgnomevfs/gnome-vfs.h>
 
 #include <string.h>
+#include <stdlib.h>
+#include <stdio.h>
 
 #include "cpufreq-monitor-procfs.h"
 #include "cpufreq-monitor-protected.h"
@@ -47,7 +50,7 @@ static CPUFreqMonitorClass *parent_class = NULL;
 
 typedef struct _CPUFreqMonitorProtected CPUFreqMonitorProtected;
 
-GType cpufreq_monitor_procfs_get_type ()
+GType cpufreq_monitor_procfs_get_type (void)
 {
 	   static GType type = 0;
 
