@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.92 2005/01/18 18:09:49 bdejean Exp $ */
+/* $Id: gweather-applet.c,v 1.94 2005/06/09 08:42:59 kmaraas Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -300,27 +300,27 @@ applet_destroy (GtkWidget *widget, GWeatherApplet *gw_applet)
 	
     if (gw_applet->gweather_info->metar_handle) {
        gnome_vfs_async_cancel(gw_applet->gweather_info->metar_handle);
-       gw_applet->gweather_info->metar_handle = 0;
+       gw_applet->gweather_info->metar_handle = NULL;
     }
 
     if (gw_applet->gweather_info->iwin_handle) {
        gnome_vfs_async_cancel(gw_applet->gweather_info->iwin_handle);
-       gw_applet->gweather_info->iwin_handle = 0;
+       gw_applet->gweather_info->iwin_handle = NULL;
     }
 
     if (gw_applet->gweather_info->wx_handle) {
        gnome_vfs_async_cancel(gw_applet->gweather_info->wx_handle);
-       gw_applet->gweather_info->wx_handle = 0;
+       gw_applet->gweather_info->wx_handle = NULL;
     }
 
     if (gw_applet->gweather_info->met_handle) {
        gnome_vfs_async_cancel(gw_applet->gweather_info->met_handle);
-       gw_applet->gweather_info->met_handle = 0;
+       gw_applet->gweather_info->met_handle = NULL;
     }
 
     if (gw_applet->gweather_info->bom_handle) {
        gnome_vfs_async_cancel(gw_applet->gweather_info->bom_handle);
-       gw_applet->gweather_info->bom_handle = 0;
+       gw_applet->gweather_info->bom_handle = NULL;
     }
 }
 
@@ -403,43 +403,6 @@ void gweather_applet_create (GWeatherApplet *gw_applet)
 	return;
 }
 
-
-void gweather_info_save (const gchar *path, GWeatherApplet *gw_applet)
-{
-    gchar *prefix;
-
-    g_return_if_fail(gw_applet->gweather_info != NULL);
-    g_return_if_fail(path != NULL);
-
-    prefix = g_strconcat (path, "WeatherInfo/", NULL);
-    gnome_config_push_prefix(prefix);
-    /* fprintf(stderr, "gweather_info_save: %s\n", prefix); */
-    g_free(prefix);
-
-    weather_info_config_write(gw_applet->gweather_info);
-
-    gnome_config_pop_prefix();
-    gnome_config_sync();
-    gnome_config_drop_all();
-}
-
-void gweather_info_load (const gchar *path, GWeatherApplet *gw_applet)
-{
-    gchar *prefix;
-
-    g_return_if_fail(path != NULL);
-
-    prefix = g_strconcat (path, "WeatherInfo/", NULL);
-    gnome_config_push_prefix(prefix);
-    /* fprintf(stderr, "gweather_info_save: %s\n", prefix); */
-    g_free(prefix);
-
-    weather_info_free (gw_applet->gweather_info);
-    gw_applet->gweather_info = weather_info_config_read(gw_applet->applet);
-
-    gnome_config_pop_prefix();
-}
-
 gint timeout_cb (gpointer data)
 {
     GWeatherApplet *gw_applet = (GWeatherApplet *)data;
